load("@ytt:data", "data")
load("@ytt:assert", "assert")
load("@ytt:struct", "struct")

# ##########
# DEFAULTING
# ##########

def get_resource_names_for_role(names):
  if names == None or len(names) == 0:
    return []
  else:
    return names.split(',')
  end
end


def get_listen_address(ipFamily):
  if ipFamily == "IPv6":
    return "::"
  else:
    return "0.0.0.0"
  end
end

def get_contour_deployment_args():
  args = [
    "serve",
    "--incluster",
    "--xds-address="+get_listen_address(data.values.contour.listenIPFamily),
    "--xds-port=8001",
    "--stats-address="+get_listen_address(data.values.envoy.listenIPFamily),
    "--http-address="+get_listen_address(data.values.contour.listenIPFamily),
    "--envoy-service-http-address="+get_listen_address(data.values.envoy.listenIPFamily),
    "--envoy-service-https-address="+get_listen_address(data.values.envoy.listenIPFamily),
    "--health-address="+get_listen_address(data.values.contour.listenIPFamily),
    "--contour-cafile=/certs/ca.crt",
    "--contour-cert-file=/certs/tls.crt",
    "--contour-key-file=/certs/tls.key",
    "--config-path=/config/contour.yaml",
    "--disable-feature=tlsroutes",
    "--disable-feature=grpcroutes",
    "--disable-feature=tcproutes",
  ]
  if data.values.contour.useProxyProtocol:
    args.append("--use-proxy-protocol")
  end

  if data.values.contour.logLevel == "debug":
    args.append("--debug")
  end

  return args
end

def get_envoy_service_type():
  if data.values.envoy.service.type:
    return data.values.envoy.service.type
  elif data.values.infrastructure_provider == "vsphere":
    return "NodePort"
  else:
    return "LoadBalancer"
  end
end

def get_envoy_service_external_traffic_policy():
  if data.values.envoy.service.externalTrafficPolicy:
    return data.values.envoy.service.externalTrafficPolicy
  elif data.values.infrastructure_provider == "vsphere":
    return "Cluster"
  else:
    return "Local"
  end
end

def get_envoy_service_annotations():
  annotations = {}

  if data.values.infrastructure_provider == "aws":
    if data.values.envoy.service.aws.LBType == "nlb":
      annotations["service.beta.kubernetes.io/aws-load-balancer-type"] = "nlb"
    else:
      annotations["service.beta.kubernetes.io/aws-load-balancer-backend-protocol"] = "tcp"
      if data.values.contour.useProxyProtocol:
        annotations["service.beta.kubernetes.io/aws-load-balancer-proxy-protocol"] = "*"
      end
    end
  end

  if data.values.envoy.service.annotations:
    annotations_kvs = struct.decode(data.values.envoy.service.annotations)
    annotations.update(annotations_kvs)
  end

  return annotations
end

# ##########
# VALIDATION
# ##########

if data.values.envoy.service.annotations:
  annotations_kvs = struct.decode(data.values.envoy.service.annotations)
  _, err = assert.try_to(lambda: annotations_kvs.items())
  not err or assert.fail("envoy.service.annotations must be a key/value map")
end

if data.values.envoy.service.loadBalancerTLSTermination == True and get_envoy_service_type() != "LoadBalancer":
  assert.fail("envoy.service.loadBalancerTLSTermination must be false when the Envoy Service Type (value: {}) is not LoadBalancer".format(get_envoy_service_type()))
end
